@@ -689,6 +689,28 @@ globus_gfs_log_result(
     GlobusGFSDebugExit();
 }
 
+void
+globus_gfs_log_exit_result(
+    const char *                        lead,
+    globus_result_t                     result)
+{
+    char *                              message;
+    GlobusGFSName(globus_gfs_log_result);
+    GlobusGFSDebugEnter();
+
+    if(result != GLOBUS_SUCCESS)
+    {
+        message = globus_error_print_friendly(globus_error_peek(result));
+    }
+    else
+    {
+        message = globus_libc_strdup("(unknown error)");
+    }
+    globus_gfs_log_exit_message("%s:\n%s\n", lead, message);
+    globus_free(message);
+
+    GlobusGFSDebugExit();
+}
 
 void
 globus_i_gfs_log_tr(
@@ -982,7 +1004,8 @@ globus_i_gfs_log_usage_stats(
     char *                              userdn,
     char *                              app,
     char *                              appver,
-    char *                              scheme)
+    char *                              scheme,
+    char *                              dsi)
 {
     time_t                              start_time_time;
     time_t                              end_time_time;
@@ -1153,9 +1176,9 @@ globus_i_gfs_log_usage_stats(
     
                   case GLOBUS_I_GFS_USAGE_DSI:
                     key = "DSI";
-                    tmp = globus_i_gfs_config_string("load_dsi_module");
+                    tmp = dsi;
                     strncpy(dsi_b, tmp, sizeof(dsi_b));
-                    dsi_b[sizeof(dsi_b - 1)] = '\0';
+                    dsi_b[sizeof(dsi_b) - 1] = '\0';
                     if((tmp = strchr(dsi_b, ':')) != NULL)
                     {
                         *tmp = '\0';
