@@ -56,7 +56,6 @@ typedef void
 
 typedef struct globus_l_gfs_file_cksm_monitor_s
 {
-
     globus_gfs_operation_t              op;
     globus_off_t                        offset;
     globus_off_t                        length;
@@ -65,9 +64,16 @@ typedef struct globus_l_gfs_file_cksm_monitor_s
     globus_size_t                       block_size;
     globus_l_gfs_file_cksm_cb_t         internal_cb;
     void *                              internal_cb_arg;
+    
+    globus_callback_handle_t            marker_handle;
+    int                                 marker_freq;
+    globus_bool_t                       send_marker;
+    globus_off_t                        total_bytes;
+    
     MD5_CTX                             mdctx;
     globus_byte_t                       buffer[1];
 } globus_l_gfs_file_cksm_monitor_t;
+
 typedef struct 
 {
     gss_cred_id_t                       cred;
@@ -698,8 +704,8 @@ globus_l_gfs_file_destroy_stat(
 }
     
 
-#define GFS_STAT_COUNT 1000
-#define GFS_STAT_COUNT_MAX 20000
+#define GFS_STAT_COUNT_CHECK 100
+#define GFS_STAT_COUNT_MAX 1000
 #define GFS_STAT_TIME 10
 
 static
@@ -778,148 +784,18 @@ globus_l_gfs_file_stat(
         stat_count = 1;
     }
     else
-#ifdef OLD_DIR_LIST
-    {
-        struct dirent *                 dir_entry;
-        int                             i;
-        char                            dir_path[MAXPATHLEN];
-    
-        stat_count = stat_info->include_path_stat ? 1 : 0;
-        
-        dir = globus_libc_opendir(stat_info->pathname);
-        if(!dir)
-        {
-            result = GlobusGFSErrorSystemError("opendir", errno);
-            if (!stat_info->include_path_stat)
-            goto error_open;
-        }
-        
-        total_stat_count = 0;
-
-        while(globus_libc_readdir_r(dir, &dir_entry) == 0 && dir_entry)
-        {
-            total_stat_count++;
-            globus_free(dir_entry);
-        }
-        
-        globus_libc_rewinddir(dir);
-        
-        stat_array = (globus_gfs_stat_t *)
-            globus_malloc(sizeof(globus_gfs_stat_t) * GLOBUS_MIN((GFS_STAT_COUNT+1), total_stat_count));
-        if(!stat_array)
-        {
-            result = GlobusGFSErrorMemory("stat_array");
-            goto error_alloc2;
-        }
-        
-        snprintf(
-            dir_path, 
-            sizeof(dir_path), 
-            "%s/%s", 
-            (basepath[0] != '/' || basepath[1] != '\0') ? basepath : "", 
-            filename);
-            
-        dir_path[MAXPATHLEN - 1] = '\0';
-        
-        i = 0;
-        if(stat_info->include_path_stat) 
-        {
-            globus_l_gfs_file_copy_stat(&stat_array[i++], &stat_buf, filename, NULL, 0,
-                dir ? GLOBUS_GRIDFTP_SERVER_CONTROL_STAT_SUCCESS : GLOBUS_GRIDFTP_SERVER_CONTROL_STAT_OPENFAILED);
-        }
-        
-        while(globus_libc_readdir_r(dir, &dir_entry) == 0 && dir_entry)
-        {
-            char                        path[MAXPATHLEN];
-                
-                base_error = GLOBUS_GRIDFTP_SERVER_CONTROL_STAT_SUCCESS;
-            snprintf(path, sizeof(path), "%s/%s", dir_path, dir_entry->d_name);
-            path[MAXPATHLEN - 1] = '\0';
-        
-            /* lstat is the same as stat when not operating on a link */
-            if(lstat(path, &stat_buf) != 0)
-            {
-                globus_free(dir_entry);
-                /* just skip invalid entries */
-                total_stat_count--;
-                continue;
-            }
-            /* if this is a link we still need to stat to get the info we are 
-                interested in and then use realpath() to get the full path of 
-                the symlink target */
-            *symlink_target = '\0';
-            if(S_ISLNK(stat_buf.st_mode))
-            {
-                int stat_result = 0;
-            
-                if(stat_info->use_symlink_info)
-                {
-                    memset(&link_stat_buf, 0, sizeof(struct stat));
-                    stat_result = stat(path, &link_stat_buf);
-                }
-                else if(stat(path, &stat_buf) != 0)
-                {
-                    globus_free(dir_entry);
-                    /* just skip invalid entries */
-                    total_stat_count--;
-                    continue;
-                }
-                if(stat_result < 0 || realpath(path, symlink_target) == NULL)
-                {
-                    int nchars = readlink(path, symlink_target, MAXPATHLEN);
-                    if (nchars < 0)
-                    {
-                        globus_free(dir_entry);
-                        /* just skip invalid entries */
-                        total_stat_count--;
-                        continue;
-                    }
-                    symlink_target[nchars] = '\0';
-                    base_error = GLOBUS_GRIDFTP_SERVER_CONTROL_STAT_INVALIDLINK;
-                }
-            }
-            
-            globus_l_gfs_file_copy_stat(
-                    &stat_array[i], &stat_buf, dir_entry->d_name, symlink_target, link_stat_buf.st_mode, base_error);
-            i++;
-            globus_free(dir_entry);
-            
-            if(i >= GFS_STAT_COUNT)
-            {
-                stat_count = i;
-                
-                globus_gridftp_server_finished_stat_partial(
-                    op, GLOBUS_SUCCESS, stat_array, stat_count);
-                    
-                globus_l_gfs_file_destroy_stat(stat_array, stat_count);
-                stat_array = (globus_gfs_stat_t *)
-                    globus_malloc(sizeof(globus_gfs_stat_t) * (GFS_STAT_COUNT+1));
-                if(!stat_array)
-                {
-                    result = GlobusGFSErrorMemory("stat_array");
-                    goto error_alloc2;
-                }
-                
-                i = 0;
-                stat_count = 0;
-            }                
-        }
-        stat_count = i;
-        
-        closedir(dir);
-    }     
-#else
     {
         struct dirent *                 dir_entry;
         struct dirent **                entries = NULL;
         int                             i;
         int                             j;
         char                            dir_path[MAXPATHLEN];
-        int                             stat_limit_count = GFS_STAT_COUNT;
+        int                             stat_limit_check = GFS_STAT_COUNT_CHECK;
+        int                             stat_limit_max = GFS_STAT_COUNT_MAX;
         time_t                          stat_limit_time;
         globus_bool_t                   check_cdir = GLOBUS_TRUE;
 
-        stat_limit_time = time(NULL);
+        stat_limit_time = time(NULL) + GFS_STAT_TIME;
         
         total_stat_count = scandir(
             stat_info->pathname, 
@@ -934,7 +810,7 @@ globus_l_gfs_file_stat(
         
         stat_array = (globus_gfs_stat_t *) globus_malloc(
             sizeof(globus_gfs_stat_t) * 
-            (GLOBUS_MIN(GFS_STAT_COUNT, total_stat_count) + 1));
+            (GLOBUS_MIN(stat_limit_max, total_stat_count) + 1));
         if(!stat_array)
         {
             result = GlobusGFSErrorMemory("stat_array");
@@ -1015,36 +891,28 @@ globus_l_gfs_file_stat(
             i++;
             globus_free(dir_entry);
             
-            /* send updates every GFS_STAT_TIME, checked every GFS_STAT_COUNT */
-            if(i >= stat_limit_count)
+            /* send updates every GFS_STAT_TIME, checked every GFS_STAT_CHECK */
+            if(i >= stat_limit_check)
             {
                 time_t                  tmp_time;
                 globus_bool_t           send_stats = GLOBUS_FALSE;
                 
                 tmp_time = time(NULL);
-                if(stat_limit_count > GFS_STAT_COUNT_MAX ||
-                    tmp_time > stat_limit_time)
+                if(i >= stat_limit_max || tmp_time > stat_limit_time)
                 {
                     send_stats = GLOBUS_TRUE;
-                    stat_limit_time = tmp_time + GFS_STAT_TIME;
                 }
                 else
                 {
-                    stat_limit_count += GFS_STAT_COUNT;
-                    stat_array = (globus_gfs_stat_t *) globus_realloc(
-                        stat_array, 
-                        sizeof(globus_gfs_stat_t) * (stat_limit_count + 1));
-                    if(!stat_array)
-                    {
-                        result = GlobusGFSErrorMemory("stat_array");
-                        goto error_alloc2;
-                    }
+                    stat_limit_check += GFS_STAT_COUNT_CHECK;
                 }
                 
                 if(send_stats)
                 {
                     stat_count = i;
-                    stat_limit_count = GFS_STAT_COUNT;
+                    stat_limit_check = GFS_STAT_COUNT_CHECK;
+                    stat_limit_time = tmp_time + GFS_STAT_TIME;
+
                     i = 0;
                     
                     globus_gridftp_server_finished_stat_partial(
@@ -1053,7 +921,7 @@ globus_l_gfs_file_stat(
                     globus_l_gfs_file_destroy_stat(stat_array, stat_count);
                     
                     stat_array = (globus_gfs_stat_t *) globus_malloc(
-                        sizeof(globus_gfs_stat_t) * (stat_limit_count + 1));
+                        sizeof(globus_gfs_stat_t) * (stat_limit_max + 1));
                     if(!stat_array)
                     {
                         result = GlobusGFSErrorMemory("stat_array");
@@ -1071,7 +939,6 @@ globus_l_gfs_file_stat(
             globus_free(entries);
         }
     }
-#endif
     
     globus_gridftp_server_finished_stat(
         op, result, stat_array, stat_count);
@@ -1518,11 +1385,23 @@ globus_l_gfs_file_cksm_read_cb(
             eof = GLOBUS_TRUE;
         }
     }
-
+    monitor->total_bytes += nbytes;
+    
     MD5_Update(&monitor->mdctx, buffer, nbytes);
 
     if(!eof)
     {
+        if(monitor->send_marker)
+        {
+            monitor->send_marker = GLOBUS_FALSE;
+            
+            char                        count[128];
+            sprintf(count, "%"GLOBUS_OFF_T_FORMAT, monitor->total_bytes);
+            
+            globus_gridftp_server_intermediate_command(
+                monitor->op, GLOBUS_SUCCESS, count);
+        }
+
         result = globus_xio_register_read(
             handle,
             monitor->buffer,
@@ -1540,6 +1419,16 @@ globus_l_gfs_file_cksm_read_cb(
     }
     else
     {
+        if(monitor->marker_handle)
+        {
+            globus_callback_unregister(
+                monitor->marker_handle,
+                NULL,
+                NULL,
+                NULL);
+            monitor->marker_handle = NULL;
+        }
+        
         MD5_Final(md, &monitor->mdctx);
     
         globus_xio_register_close(
@@ -1586,6 +1475,15 @@ error_read:
 }
 
 
+static void
+globus_l_gfs_file_marker_cb(
+    void *                              user_arg)
+{
+    globus_l_gfs_file_cksm_monitor_t *  monitor;
+    
+    monitor = (globus_l_gfs_file_cksm_monitor_t *) user_arg;
+    monitor->send_marker = GLOBUS_TRUE;
+}
 
 static
 void
@@ -1595,6 +1493,7 @@ globus_l_gfs_file_open_cksm_cb(
     void *                              user_arg)
 {  
     globus_l_gfs_file_cksm_monitor_t *  monitor;
+    char *                              freq;
     GlobusGFSName(globus_l_gfs_file_open_cksm_cb);
     GlobusGFSFileDebugEnter();
     
@@ -1607,6 +1506,30 @@ globus_l_gfs_file_open_cksm_cb(
         goto error_open;  
     }  
     
+    if(monitor->op)
+    {
+        globus_gridftp_server_get_update_interval(
+            monitor->op, &monitor->marker_freq);
+    }
+
+    if(monitor->marker_freq)
+    {
+        globus_result_t                     res;
+        globus_reltime_t                    delay;
+        
+        GlobusTimeReltimeSet(delay, monitor->marker_freq, 0);
+        res = globus_callback_register_periodic(
+            &monitor->marker_handle,
+            &delay,
+            &delay,
+            globus_l_gfs_file_marker_cb,
+            monitor);
+        if(res != GLOBUS_SUCCESS)
+        {
+            
+        }
+    }
+    
     if(monitor->length >= 0)
     {
         monitor->read_left = monitor->length;
@@ -3068,7 +2991,7 @@ globus_l_gfs_file_deactivate(void);
 
 static globus_gfs_storage_iface_t       globus_l_gfs_file_dsi_iface = 
 {
-    GLOBUS_GFS_DSI_DESCRIPTOR_SENDER,
+    GLOBUS_GFS_DSI_DESCRIPTOR_SENDER | GLOBUS_GFS_DSI_DESCRIPTOR_HAS_REALPATH,
     globus_l_gfs_file_init,
     globus_l_gfs_file_destroy,
     NULL, /* list */
